package com.example.demo.users.api;

import java.util.Map;

import org.modelmapper.ModelMapper;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import com.example.demo.core.api.PageAttributesMapper;
import com.example.demo.core.configuration.Constants;
import com.example.demo.users.model.UserEntity;
import com.example.demo.users.service.UserService;

import jakarta.validation.Valid;

// Контроллер для сущности "Пользователь"
@Controller
@RequestMapping(UserController.URL)
public class UserController {
    // URL для доступа к методам контроллера
    public static final String URL = Constants.ADMIN_PREFIX + "/user";

    // Представление для отображения списка пользователей
    private static final String USER_VIEW = "user";

    // Представление для создания или редактирования пользователя
    private static final String USER_EDIT_VIEW = "user-edit";

    // Атрибут модели для обработки данных
    private static final String USER_ATTRIBUTE = "user";

    // Атрибут модели для пагинации
    private static final String PAGE_ATTRIBUTE = "page";

    // Бизнес-логика для сущности "Пользователь"
    private final UserService userService;

    // Библиотека для преобразования сущности
    private final ModelMapper modelMapper;

    // Конструктор
    public UserController(UserService userService, ModelMapper modelMapper) {
        this.userService = userService;
        this.modelMapper = modelMapper;
    }

    // Преобразовать из сущности в DTO
    private UserDto toDto(UserEntity entity) {
        return modelMapper.map(entity, UserDto.class);
    }

    // Преобразовать из DTO в сущность
    private UserEntity toEntity(@Valid UserDto dto) {
        return modelMapper.map(dto, UserEntity.class);
    }

    // Получить все элементы
    @GetMapping
    public String getAll(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        final Map<String, Object> attributes = PageAttributesMapper.toAttributes(
                "user",
                userService.getAll(page, Constants.DEFAULT_PAGE_SIZE), this::toDto);
        model.addAllAttributes(attributes);
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return USER_VIEW;
    }

    // Создать элемент
    @GetMapping("/edit/")
    public String create(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        model.addAttribute(USER_ATTRIBUTE, new UserDto());
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return USER_EDIT_VIEW;
    }

    // Создать элемент
    @PostMapping("/edit/")
    public String create(
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserDto user,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return USER_EDIT_VIEW;
        }

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        userService.create(toEntity(user));
        return Constants.REDIRECT_VIEW + URL;
    }

    // Изменить элемент
    @GetMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            Model model) {
        if (id <= 0) {
            throw new IllegalArgumentException();
        }

        model.addAttribute(USER_ATTRIBUTE, toDto(userService.get(id)));
        model.addAttribute(PAGE_ATTRIBUTE, page);
        return USER_EDIT_VIEW;
    }

    // Изменить элемент
    @PostMapping("/edit/{id}")
    public String update(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            @ModelAttribute(name = USER_ATTRIBUTE) @Valid UserDto user,
            BindingResult bindingResult,
            Model model,
            RedirectAttributes redirectAttributes) {
        if (bindingResult.hasErrors()) {
            model.addAttribute(PAGE_ATTRIBUTE, page);
            return USER_EDIT_VIEW;
        }
        if (id <= 0) {
            throw new IllegalArgumentException();
        }

        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        userService.update(id, toEntity(user));
        return Constants.REDIRECT_VIEW + URL;
    }

    // Удалить элемент
    @PostMapping("/delete/{id}")
    public String delete(
            @PathVariable(name = "id") Long id,
            @RequestParam(name = PAGE_ATTRIBUTE, defaultValue = "0") int page,
            RedirectAttributes redirectAttributes) {
        redirectAttributes.addAttribute(PAGE_ATTRIBUTE, page);
        userService.delete(id);
        return Constants.REDIRECT_VIEW + URL;
    }
}
